/* gnomeProxy.c - a plugin for xchat
 * Copyright (C) 2010 Andrea Marchesini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <string.h>

#include <gconf/gconf-client.h>
#include <xchat/xchat-plugin.h>

#define PNAME "GnomeProxy"
#define PDESC "Use the gnome settings for proxy"

/* Settings:
 * net_proxy_host	Proxy host to use.
 * net_proxy_port	Port to use for proxy host.
 * net_proxy_type	Type of proxy to use.
o
 * 			1=Wingate
 * 			2=Socks4
 * 			3=Socks5
 * 			4=HTTP
 * 			5=MS Proxy (ISA)
 * net_proxy_use	What to use proxies for (if set).
 * 			0=All
 * 			1=IRC Only
 * 			2=DCC Only
 * net_proxy_auth	Toggle proxy authentication.
 * net_proxy_user	Username to use if proxy authentication is turned on.
 * net_proxy_pass	Password to use if proxy authentication is turned on.
 */
typedef struct gconf_proxy_t GConfProxy;
struct gconf_proxy_t
{
  gboolean initialized;

  gchar    host[1024];
  gint     port;
  gint     type;
  gboolean auth;
  gchar    user[1024];
  gchar    pass[1024];
};

/* Some static variable: */
static xchat_plugin *ph;
static GConfProxy proxyConf = { FALSE };

/* Main function: */
int gconf_proxy(GConfClient *client)
{
  GConfProxy proxy;
  gchar *str;

  /* I create a temporary GConfProxy struct, then I compare this with
   * the global one (proxyConf): */
  memset(&proxy, 0, sizeof(GConfProxy));
  proxy.initialized = TRUE;

  str = gconf_client_get_string(client, "/system/proxy/mode", NULL);

  /* No Proxy: */
  if (!str || !strcmp(str, "none")) {
    proxy.type = 0;

    if (str)
      g_free(str);
  }

  /* HTTP or SOCKS PRoxy: */
  else {
    g_free(str);

    /* HTTP Authentication: */
    if (!gconf_client_get_bool(client, "/system/http_proxy/use_authentication", NULL)) {
      proxy.auth = 0;
    } else {
      proxy.auth = 1;

      if ((str = gconf_client_get_string(client, "/system/http_proxy/authentication_user", NULL))) {
        if(*str)
          strncpy(proxy.user, str, sizeof(proxy.user));

        g_free(str);
      }

      if ((str = gconf_client_get_string(client, "/system/http_proxy/authentication_password", NULL))) {
        if(*str)
          strncpy(proxy.pass, str, sizeof(proxy.pass));

        g_free(str);
      }
    }

    /* HTTP Proxy: */
    if ((str = gconf_client_get_string(client, "/system/http_proxy/host", NULL))) {
      if(*str) {
        proxy.type = 4;

        strncpy(proxy.host, str, sizeof(proxy.host));

        if (!(proxy.port = gconf_client_get_int(client, "/system/http_proxy/port", NULL)))
          proxy.port = 8080;
      }

      g_free(str);
    }

    /* SOCKS Proxy: */
    if (proxy.type == 0 && (str = gconf_client_get_string(client, "/system/proxy/socks_host", NULL))) {
      if(*str) {
        proxy.type = 3; /* or 2... witch kind of socket shoud I use? */

        strncpy(proxy.host, str, sizeof(proxy.host));

        if (!(proxy.port = gconf_client_get_int(client, "/system/proxy/socks_port", NULL)))
          proxy.port = 8080;
      }

      g_free(str);
    }
  }

  /* Comparing with the previous settings: */
  if(memcmp(&proxyConf, &proxy, sizeof(GConfProxy))) {
    gchar str[1024];

    memcpy(&proxyConf, &proxy, sizeof(GConfProxy));

    /* Proxy Type: */
    snprintf(str, sizeof(str), "set net_proxy_type %d", proxy.type);
    xchat_command(ph, str);

    /* Use proxy everywhere: */
    xchat_command(ph, "set net_proxy_use 0");

    /* HTTP Authentication: */
    snprintf(str, sizeof(str), "set net_proxy_auth %d", proxy.auth);
    xchat_command(ph, str);

    /* HTTP Authentication Username: */
    if(*proxy.user) {
      snprintf(str, sizeof(str), "set net_proxy_user %s", proxy.user);
      xchat_command(ph, str);
    }

    /* HTTP Authentication Password: */
    if(*proxy.pass) {
      snprintf(str, sizeof(str), "set net_proxy_pass %s", proxy.pass);
      xchat_command(ph, str);
    }

    /* Proxy Host: */
    if(*proxy.host) {
      snprintf(str, sizeof(str), "set net_proxy_host %s", proxy.host);
      xchat_command(ph, str);
    }

    /* Proxy Port: */
    if(proxy.port) {
      snprintf(str, sizeof(str), "set net_proxy_port %d", proxy.port);
      xchat_command(ph, str);
    }
  }

  return 1;
}

void xchat_plugin_get_info(char **name, char **desc, char **version, void **reserved)
{
  *name = PNAME;
  *desc = PDESC;
  *version = PVERSION;
}

int xchat_plugin_init(xchat_plugin *plugin_handle,
                      char **plugin_name,
                      char **plugin_desc,
                      char **plugin_version,
                      char *arg)
{
  GConfClient *client;

  /* we need to save this for use with any xchat_* functions */
  ph = plugin_handle;

  xchat_plugin_get_info(plugin_name, plugin_desc, plugin_version, NULL);
  xchat_print(plugin_handle, PNAME " is setting the configuration...\n");

  /* GConf + a first loop of configuration: */
  gconf_init(0, NULL, NULL);
  client = gconf_client_get_default();
  gconf_proxy(client);

  /* Any 2 seconds, I check if something is changed:
     (xchat plugins do not support glib main loop) */
  xchat_hook_timer(ph, 2000, (int (*)(void *)) gconf_proxy, client);

  xchat_print(plugin_handle, PNAME " loaded successfully!\n");

  /* return 1 for success */
  return 1;
}
